model;
# firm.mod
#
# Price consistent version of the MPEC.  Because the objective is not
# separable in the arbitrager variables, the NCP is not square.  
#
# the resulting problem is a nonsquare complementarity problem!
#
# original by S. Leyffer, Argonne Natl. Lab., Nov. 2002.
# modified by T. Munson
#

### set definitions
set N;				# set of nodes in network
set F;				# set of firms (generators)
set ARCS in N cross N;

### parameters & constants
param c{F,N} default 0;		# cost per unit generation at node n by firm f
param C{F,N};			# production capacity at node n for firm f
param e{ARCS};			# ISO's unit cost of shipping along arcs

param P0{N};			# price intercept of sales function at node n
param Q0{N};			# quatity intercept of sales function at node n

### variables (one copy for each firm)
var  s{F,ARCS} >= 0;		# amount produced by f at node n1, sold at n2
var ss{ARCS} >= 0, :=1;		# slacks for easier complementarity
var  a{ARCS}   >= 0, :=1;	# amount bought by arbitrager at n1, sold at n2 
var  w{ARCS};			# unit charge of shipment received by ISO

### multipliers (each firm has different multipliers)
var l_slk{ARCS};
var l_cap{F,N} >= 0;		# inequality constraints
var l_cmp   >= 0 := 1;

### defined variables 
var St{j in N}	 		# total sales at node n
    =   sum{(i,j) in ARCS} (a[i,j] + sum{t in F} s[t,i,j]) 
      - sum{(j,i) in ARCS} (a[j,i]);
var Sf{f in F, j in N}		# total sales for firm f at node n
    =   sum{(i,j) in ARCS} (a[i,j] + s[f,i,j])
      - sum{(j,i) in ARCS} (a[j,i]);

subject to

### first order equations wrt s[f,i,j]
dobj_ds {f in F, (i,j) in ARCS}: 
    0 <=   P0[j]/Q0[j]*(St[j] + Sf[f,j]) - P0[j] + w[i,j] + c[f,i] 
         + l_cap[f,i] 
         - P0[j]/Q0[j]*(sum{(k,j) in ARCS} l_slk[k,j] - 
		        sum{(j,k) in ARCS} l_slk[j,k])
    complements   s[f,i,j] >= 0;

### first order equations wrt a[i,j]
dobj_da {f in F, (i,j) in ARCS}:
    0 <=   P0[j]/Q0[j]*(St[j] + Sf[f,j]) - P0[j]
         - P0[i]/Q0[i]*(St[i] + Sf[f,i]) + P0[i] + w[i,j] 
         - P0[j]/Q0[j]*(sum{(k,j) in ARCS} l_slk[k,j] - 
                        sum{(j,k) in ARCS} l_slk[j,k]) 
         + P0[i]/Q0[i]*(sum{(k,i) in ARCS} l_slk[k,i] - 
                        sum{(i,k) in ARCS} l_slk[i,k]) 
         + ss[i,j]*l_cmp
    complements   a[i,j] >= 0;

### first order equations wrt ss[f,i,j]
dobj_dss {(i,j) in ARCS}:
    0 <= l_slk[i,j] + a[i,j]*l_cmp
    complements   ss[i,j] >= 0;
    
### capacity constraint
cap{f in F, i in N}: 
    0 <= C[f,i] - sum{(i,j) in ARCS} s[f,i,j]
    complements   l_cap[f,i] >= 0;

### define slacks for complementarity
slk{(i,j) in ARCS}: 
    (P0[i] - P0[i]/Q0[i]*St[i]) + w[i,j] 
  - (P0[j] - P0[j]/Q0[j]*St[j]) = ss[i,j]
    complements   l_slk[i,j];

### arbitrager's optimality conditions (follower)
cmp: 
    0 <= - sum{(i,j) in ARCS} a[i,j]*ss[i,j] 
    complements l_cmp >= 0;

### ISO's optimality conditions
iso{(i,j) in ARCS}: 
    sum{f in F} (s[f,i,j] + a[i,j]/card(F)) >= 0  
    complements   w[i,j] <= e[i,j];

